package com.rectanglel.patstatic.mock;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.rectanglel.patstatic.model.PatApiService;
import com.rectanglel.patstatic.model.RetrofitPatApi;
import com.rectanglel.patstatic.patterns.response.PatternResponse;
import com.rectanglel.patstatic.patterns.response.Ptr;

import java.util.List;

import io.reactivex.Flowable;
import io.reactivex.Observable;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * <p>Sets up a mock object for the {@link PatApiService}</p>
 * <p>Created by epicstar on 7/18/16.</p>
 * @author Jeremy Jao
 */
public class PatApiMock {

    public final static String testRoute1 = "71A";
    public final static String testRoute2 = "71B";
    public final static String stubApiKey = "223423645dsfsw35";

    private final static String json = "{        'bustime-response': {            'ptr': [            {                'pid': 4683,                    'ln': 35044.0,                    'rtdir': 'INBOUND',                    'pt': [                {                    'seq': 1,                        'lat': 40.479130917836,                        'lon': -79.918946109786,                        'typ': 'S',                        'stpid': '3118',                        'stpnm': 'St Clair St at Bunkerhill Apts',                        'pdist': 0.0                },                {                    'seq': 2,                        'lat': 40.47767577138,                        'lon': -79.919823282409,                        'typ': 'S',                        'stpid': '3119',                        'stpnm': 'St Clair St at Callowhill St',                        'pdist': 494.0                },                {                    'seq': 3,                        'lat': 40.476123595525,                        'lon': -79.920766675266,                        'typ': 'S',                        'stpid': '3120',                        'stpnm': 'St Clair St at Bryant St',                        'pdist': 1070.0                },                {                    'seq': 4,                        'lat': 40.47644814097,                        'lon': -79.92160233069,                        'typ': 'S',                        'stpid': '8275',                        'stpnm': 'Bryant St at Mellon St',                        'pdist': 1338.0                },                {                    'seq': 5,                        'lat': 40.477222926765,                        'lon': -79.923650059526,                        'typ': 'S',                        'stpid': '8176',                        'stpnm': 'Bryant St at Negley Ave',                        'pdist': 1975.0                },                {                    'seq': 6,                        'lat': 40.477271892988,                        'lon': -79.923778805558,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 7,                        'lat': 40.476320192708,                        'lon': -79.924169208332,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 8,                        'lat': 40.475589765811,                        'lon': -79.924507166668,                        'typ': 'S',                        'stpid': '8177',                        'stpnm': 'Negley Ave at Hampton St',                        'pdist': 2657.0                },                {                    'seq': 9,                        'lat': 40.474256571225,                        'lon': -79.925073968915,                        'typ': 'S',                        'stpid': '8178',                        'stpnm': 'Negley Ave opp Wellesley Ave',                        'pdist': 3140.0                },                {                    'seq': 10,                        'lat': 40.472544644863,                        'lon': -79.925840646824,                        'typ': 'S',                        'stpid': '8179',                        'stpnm': 'Negley Ave at Jackson St',                        'pdist': 3807.0                },                {                    'seq': 11,                        'lat': 40.471702432686,                        'lon': -79.926223833333,                        'typ': 'S',                        'stpid': '8180',                        'stpnm': 'Negley Ave at Stanton Ave',                        'pdist': 4156.0                },                {                    'seq': 12,                        'lat': 40.469921099448,                        'lon': -79.927024833332,                        'typ': 'S',                        'stpid': '8181',                        'stpnm': 'Negley Ave at Hays St',                        'pdist': 4837.0                },                {                    'seq': 13,                        'lat': 40.468833222028,                        'lon': -79.927500895503,                        'typ': 'S',                        'stpid': '8182',                        'stpnm': 'Negley Ave at Black St',                        'pdist': 5230.0                },                {                    'seq': 14,                        'lat': 40.467832036418,                        'lon': -79.928103728835,                        'typ': 'S',                        'stpid': '8183',                        'stpnm': 'Negley Ave at Margaretta St',                        'pdist': 5636.0                },                {                    'seq': 15,                        'lat': 40.466869266278,                        'lon': -79.928684937832,                        'typ': 'S',                        'stpid': '8184',                        'stpnm': 'Negley Ave opp Rippey St (farside)',                        'pdist': 6035.0                },                {                    'seq': 16,                        'lat': 40.465706014227,                        'lon': -79.929386011242,                        'typ': 'S',                        'stpid': '8185',                        'stpnm': 'Negley Ave opp Rural St',                        'pdist': 6377.0                },                {                    'seq': 17,                        'lat': 40.464633933067,                        'lon': -79.930061333333,                        'typ': 'S',                        'stpid': '8186',                        'stpnm': 'Negley Ave at Broad St',                        'pdist': 6930.0                },                {                    'seq': 18,                        'lat': 40.463050518435,                        'lon': -79.930969782409,                        'typ': 'S',                        'stpid': '19060',                        'stpnm': 'Negley Ave past Penn Ave',                        'pdist': 7491.0                },                {                    'seq': 19,                        'lat': 40.462015444544,                        'lon': -79.93160924008,                        'typ': 'S',                        'stpid': '8188',                        'stpnm': 'Negley Ave at Coral St',                        'pdist': 7963.0                },                {                    'seq': 20,                        'lat': 40.460807766604,                        'lon': -79.932365500001,                        'typ': 'S',                        'stpid': '8189',                        'stpnm': 'Negley Ave at Friendship Ave',                        'pdist': 8463.0                },                {                    'seq': 21,                        'lat': 40.459079351742,                        'lon': -79.933393646824,                        'typ': 'S',                        'stpid': '8190',                        'stpnm': 'Negley Ave at #370',                        'pdist': 9160.0                },                {                    'seq': 22,                        'lat': 40.457797710009,                        'lon': -79.93416612302,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 23,                        'lat': 40.45757729782,                        'lon': -79.9342197672,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 24,                        'lat': 40.457299991305,                        'lon': -79.934121398147,                        'typ': 'S',                        'stpid': '8191',                        'stpnm': 'Negley Ave at Centre Ave',                        'pdist': 9795.0                },                {                    'seq': 25,                        'lat': 40.456588788684,                        'lon': -79.936404395502,                        'typ': 'S',                        'stpid': '8192',                        'stpnm': 'Centre Ave at Graham St',                        'pdist': 10499.0                },                {                    'seq': 26,                        'lat': 40.45594147008,                        'lon': -79.938546226853,                        'typ': 'S',                        'stpid': '8193',                        'stpnm': 'Centre Ave at Aiken Ave',                        'pdist': 11045.0                },                {                    'seq': 27,                        'lat': 40.455471658241,                        'lon': -79.940120982802,                        'typ': 'S',                        'stpid': '8194',                        'stpnm': 'Centre Ave opp Shadyside Hospital',                        'pdist': 11676.0                },                {                    'seq': 28,                        'lat': 40.455069439451,                        'lon': -79.941432683864,                        'typ': 'S',                        'stpid': '8195',                        'stpnm': 'Centre Ave at Cypress St',                        'pdist': 11993.0                },                {                    'seq': 29,                        'lat': 40.454661251586,                        'lon': -79.942848890224,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 30,                        'lat': 40.454203439285,                        'lon': -79.94441278836,                        'typ': 'S',                        'stpid': '8196',                        'stpnm': 'Centre Ave at Morewood Ave',                        'pdist': 12883.0                },                {                    'seq': 31,                        'lat': 40.453325441844,                        'lon': -79.947320525796,                        'typ': 'S',                        'stpid': '8197',                        'stpnm': 'Centre Ave at Millvale Ave',                        'pdist': 13737.0                },                {                    'seq': 32,                        'lat': 40.452851703376,                        'lon': -79.94905713492,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 33,                        'lat': 40.452672095094,                        'lon': -79.949663314157,                        'typ': 'S',                        'stpid': '2631',                        'stpnm': 'Centre Ave opp Neville St',                        'pdist': 14405.0                },                {                    'seq': 34,                        'lat': 40.452286780054,                        'lon': -79.950813788362,                        'typ': 'S',                        'stpid': '2632',                        'stpnm': 'Centre Ave at Melwood Ave',                        'pdist': 14787.0                },                {                    'seq': 35,                        'lat': 40.451902128516,                        'lon': -79.952080893522,                        'typ': 'S',                        'stpid': '2633',                        'stpnm': 'Centre Ave past Craig St ',                        'pdist': 15265.0                },                {                    'seq': 36,                        'lat': 40.451840897608,                        'lon': -79.952037978177,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 37,                        'lat': 40.449890100525,                        'lon': -79.950836166666,                        'typ': 'S',                        'stpid': '2634',                        'stpnm': 'Craig St at Bayard St',                        'pdist': 15930.0                },                {                    'seq': 38,                        'lat': 40.447257934,                        'lon': -79.949188166667,                        'typ': 'S',                        'stpid': '2635',                        'stpnm': 'Craig St at Fifth Ave',                        'pdist': 17002.0                },                {                    'seq': 39,                        'lat': 40.446951757745,                        'lon': -79.949064785052,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 40,                        'lat': 40.446853781049,                        'lon': -79.95005183797,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 41,                        'lat': 40.446796627911,                        'lon': -79.951269560863,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 42,                        'lat': 40.446723145233,                        'lon': -79.951559239436,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 43,                        'lat': 40.446598625546,                        'lon': -79.951805621695,                        'typ': 'S',                        'stpid': '1171',                        'stpnm': 'Fifth Ave at Bellefield Ave',                        'pdist': 17809.0                },                {                    'seq': 44,                        'lat': 40.445674281789,                        'lon': -79.953128988756,                        'typ': 'S',                        'stpid': '33',                        'stpnm': 'Fifth Ave at Tennyson Ave',                        'pdist': 18336.0                },                {                    'seq': 45,                        'lat': 40.444567437463,                        'lon': -79.954665240081,                        'typ': 'S',                        'stpid': '34',                        'stpnm': 'Fifth Ave at Bigelow Blvd',                        'pdist': 18945.0                },                {                    'seq': 46,                        'lat': 40.443287941739,                        'lon': -79.956426053571,                        'typ': 'S',                        'stpid': '35',                        'stpnm': 'Fifth Ave at Thackeray Ave',                        'pdist': 19631.0                },                {                    'seq': 47,                        'lat': 40.441797374239,                        'lon': -79.958504737436,                        'typ': 'S',                        'stpid': '36',                        'stpnm': 'Fifth Ave opp Atwood St',                        'pdist': 20382.0                },                {                    'seq': 48,                        'lat': 40.439498767751,                        'lon': -79.961631833334,                        'typ': 'S',                        'stpid': '38',                        'stpnm': 'Fifth Ave at Chesterfield Rd',                        'pdist': 21647.0                },                {                    'seq': 49,                        'lat': 40.438439620008,                        'lon': -79.963037673274,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 50,                        'lat': 40.438194646982,                        'lon': -79.963327351848,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 51,                        'lat': 40.43796192178,                        'lon': -79.9637296832,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 52,                        'lat': 40.43786393198,                        'lon': -79.964008632937,                        'typ': 'S',                        'stpid': '2639',                        'stpnm': 'Fifth Ave opp Craft Ave',                        'pdist': 22501.0                },                {                    'seq': 53,                        'lat': 40.437761859121,                        'lon': -79.964722100536,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 54,                        'lat': 40.437786356621,                        'lon': -79.965043965617,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 55,                        'lat': 40.437783509968,                        'lon': -79.965337675265,                        'typ': 'S',                        'stpid': '2640',                        'stpnm': 'Fifth Ave at Robinson St',                        'pdist': 22888.0                },                {                    'seq': 56,                        'lat': 40.437501788161,                        'lon': -79.965552251986,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 57,                        'lat': 40.437269060561,                        'lon': -79.965616625003,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 58,                        'lat': 40.437056746959,                        'lon': -79.965766828707,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 59,                        'lat': 40.436913843196,                        'lon': -79.965874117068,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 60,                        'lat': 40.436762773174,                        'lon': -79.966045778445,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 61,                        'lat': 40.436615785799,                        'lon': -79.966260355166,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 62,                        'lat': 40.43652187703,                        'lon': -79.966560762576,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 63,                        'lat': 40.436472881098,                        'lon': -79.966989916018,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 64,                        'lat': 40.436465920651,                        'lon': -79.968430881616,                        'typ': 'S',                        'stpid': '18165',                        'stpnm': 'Fifth Ave past Brenham St',                        'pdist': 23993.0                },                {                    'seq': 65,                        'lat': 40.436457754658,                        'lon': -79.968661551591,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 66,                        'lat': 40.436484215649,                        'lon': -79.968868819433,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 67,                        'lat': 40.436925177673,                        'lon': -79.97015627976,                        'typ': 'S',                        'stpid': '2643',                        'stpnm': '5th Ave opp #2410',                        'pdist': 24457.0                },                {                    'seq': 68,                        'lat': 40.437194762564,                        'lon': -79.970976146824,                        'typ': 'S',                        'stpid': '2644',                        'stpnm': '5th Ave opp #2358',                        'pdist': 24728.0                },                {                    'seq': 69,                        'lat': 40.437822924516,                        'lon': -79.973009164022,                        'typ': 'S',                        'stpid': '18164',                        'stpnm': 'Fifth Ave past Kirkpatrick St',                        'pdist': 25322.0                },                {                    'seq': 70,                        'lat': 40.437953577631,                        'lon': -79.973400766539,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 71,                        'lat': 40.438046432767,                        'lon': -79.974464949074,                        'typ': 'S',                        'stpid': '3147',                        'stpnm': '5th Ave past Moultrie',                        'pdist': 25724.0                },                {                    'seq': 72,                        'lat': 40.438166929226,                        'lon': -79.975773604499,                        'typ': 'S',                        'stpid': '3148',                        'stpnm': 'Fifth Ave opp Seneca St',                        'pdist': 26087.0                },                {                    'seq': 73,                        'lat': 40.438247594947,                        'lon': -79.977394310846,                        'typ': 'S',                        'stpid': '3149',                        'stpnm': 'Fifth Ave at Wyandotte St',                        'pdist': 26526.0                },                {                    'seq': 74,                        'lat': 40.438339593968,                        'lon': -79.97944159061,                        'typ': 'S',                        'stpid': '3150',                        'stpnm': 'Fifth Ave opp Gist St',                        'pdist': 27085.0                },                {                    'seq': 75,                        'lat': 40.438395013737,                        'lon': -79.980569064815,                        'typ': 'S',                        'stpid': '3151',                        'stpnm': 'Fifth Ave at Dinwiddie St',                        'pdist': 27449.0                },                {                    'seq': 76,                        'lat': 40.438484094732,                        'lon': -79.982623861111,                        'typ': 'S',                        'stpid': '3152',                        'stpnm': 'Fifth Ave opp Van Braam St',                        'pdist': 28063.0                },                {                    'seq': 77,                        'lat': 40.438582096493,                        'lon': -79.984903019843,                        'typ': 'S',                        'stpid': '3154',                        'stpnm': 'Fifth Ave at Pride St',                        'pdist': 28624.0                },                {                    'seq': 78,                        'lat': 40.438632511731,                        'lon': -79.986305788361,                        'typ': 'S',                        'stpid': '3155',                        'stpnm': 'Fifth Ave at Stevenson St',                        'pdist': 28999.0                },                {                    'seq': 79,                        'lat': 40.438710016375,                        'lon': -79.988097240079,                        'typ': 'S',                        'stpid': '3156',                        'stpnm': 'Fifth Ave at Magee St',                        'pdist': 29532.0                },                {                    'seq': 80,                        'lat': 40.43883051923,                        'lon': -79.990663968915,                        'typ': 'S',                        'stpid': '3157',                        'stpnm': 'Fifth Ave at Washington Pl',                        'pdist': 30251.0                },                {                    'seq': 81,                        'lat': 40.43889764138,                        'lon': -79.992593541,                        'typ': 'S',                        'stpid': '18161',                        'stpnm': 'Fifth Ave opp Diamond St',                        'pdist': 30836.0                },                {                    'seq': 82,                        'lat': 40.438975215457,                        'lon': -79.99466420636,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 83,                        'lat': 40.439685626527,                        'lon': -79.994224324081,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 84,                        'lat': 40.439787696466,                        'lon': -79.994181408737,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 85,                        'lat': 40.439906541484,                        'lon': -79.994164892194,                        'typ': 'S',                        'stpid': '20292',                        'stpnm': 'Sixth Ave at Center Ave',                        'pdist': 31676.0                },                {                    'seq': 86,                        'lat': 40.44002494221,                        'lon': -79.994191714284,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 87,                        'lat': 40.440151508273,                        'lon': -79.994266816137,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 88,                        'lat': 40.440265825802,                        'lon': -79.994379468915,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 89,                        'lat': 40.440377273463,                        'lon': -79.994631135581,                        'typ': 'S',                        'stpid': '1509',                        'stpnm': 'Sixth Ave at Bigelow Blvd',                        'pdist': 31967.0                },                {                    'seq': 90,                        'lat': 40.440657024606,                        'lon': -79.995568611756,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 91,                        'lat': 40.441420492964,                        'lon': -79.997612455025,                        'typ': 'S',                        'stpid': '3158',                        'stpnm': '6th Ave at Smithfield St',                        'pdist': 32849.0                },                {                    'seq': 92,                        'lat': 40.442103148627,                        'lon': -79.999358695108,                        'typ': 'S',                        'stpid': '3159',                        'stpnm': '6th Ave at Wood St',                        'pdist': 33383.0                },                {                    'seq': 93,                        'lat': 40.442264429979,                        'lon': -79.999753832817,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 94,                        'lat': 40.442337917532,                        'lon': -79.999845027924,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 95,                        'lat': 40.442468561872,                        'lon': -79.999898672104,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 96,                        'lat': 40.442000129435,                        'lon': -80.001776661376,                        'typ': 'S',                        'stpid': '22256',                        'stpnm': 'Liberty Ave at 6th St (nearside)',                        'pdist': 34004.0                },                {                    'seq': 97,                        'lat': 40.441721436127,                        'lon': -80.002774000168,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 98,                        'lat': 40.441635699855,                        'lon': -80.002736449242,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 99,                        'lat': 40.441509136586,                        'lon': -80.002537965775,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 100,                        'lat': 40.441349911491,                        'lon': -80.00216782093,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 101,                        'lat': 40.44088273536,                        'lon': -80.000846662042,                        'typ': 'S',                        'stpid': '20690',                        'stpnm': 'Fifth Ave at Wood St',                        'pdist': 35044.0                }                ]            },            {                'pid': 2358,                    'ln': 34633.0,                    'rtdir': 'OUTBOUND',                    'pt': [                {                    'seq': 1,                        'lat': 40.44088273536,                        'lon': -80.000846662042,                        'typ': 'S',                        'stpid': '20690',                        'stpnm': 'Fifth Ave at Wood St',                        'pdist': 0.0                },                {                    'seq': 2,                        'lat': 40.440076181611,                        'lon': -79.998723841932,                        'typ': 'S',                        'stpid': '20691',                        'stpnm': 'Fifth Ave at Smithfield St',                        'pdist': 545.0                },                {                    'seq': 3,                        'lat': 40.439031583534,                        'lon': -79.995996042329,                        'typ': 'S',                        'stpid': '20293',                        'stpnm': 'Fifth Ave at Ross St',                        'pdist': 1458.0                },                {                    'seq': 4,                        'lat': 40.438938169231,                        'lon': -79.995763922636,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 5,                        'lat': 40.438925920691,                        'lon': -79.995597625677,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 6,                        'lat': 40.43888917506,                        'lon': -79.994809056227,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 7,                        'lat': 40.438803435176,                        'lon': -79.994809056227,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 8,                        'lat': 40.437999108078,                        'lon': -79.994369173948,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 9,                        'lat': 40.437999108078,                        'lon': -79.993441129629,                        'typ': 'S',                        'stpid': '3241',                        'stpnm': 'Forbes Ave at Boyd St',                        'pdist': 2315.0                },                {                    'seq': 10,                        'lat': 40.437917437913,                        'lon': -79.991346486111,                        'typ': 'S',                        'stpid': '3242',                        'stpnm': 'Forbes Ave at McAnulty Dr',                        'pdist': 2928.0                },                {                    'seq': 11,                        'lat': 40.437810426188,                        'lon': -79.9887468168,                        'typ': 'S',                        'stpid': '3243',                        'stpnm': 'Forbes Ave at Magee St',                        'pdist': 3704.0                },                {                    'seq': 12,                        'lat': 40.437725178342,                        'lon': -79.986794570767,                        'typ': 'S',                        'stpid': '3244',                        'stpnm': 'Forbes Ave at Stevenson St',                        'pdist': 4213.0                },                {                    'seq': 13,                        'lat': 40.437646432071,                        'lon': -79.98534843783,                        'typ': 'S',                        'stpid': '3245',                        'stpnm': 'Forbes Ave at Pride St',                        'pdist': 4590.0                },                {                    'seq': 14,                        'lat': 40.437541934523,                        'lon': -79.982940666667,                        'typ': 'S',                        'stpid': '3246',                        'stpnm': 'Forbes Ave at Van Braam St',                        'pdist': 5280.0                },                {                    'seq': 15,                        'lat': 40.437488434527,                        'lon': -79.981758166667,                        'typ': 'S',                        'stpid': '3247',                        'stpnm': 'Forbes Ave at Miltenberger St',                        'pdist': 5601.0                },                {                    'seq': 16,                        'lat': 40.43739501512,                        'lon': -79.979675333334,                        'typ': 'S',                        'stpid': '3248',                        'stpnm': 'Forbes Ave at Gist St',                        'pdist': 6182.0                },                {                    'seq': 17,                        'lat': 40.437313356274,                        'lon': -79.977685134245,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 18,                        'lat': 40.437740517434,                        'lon': -79.977664844577,                        'typ': 'S',                        'stpid': '3249',                        'stpnm': 'Jumonville St at Watson St',                        'pdist': 6840.0                },                {                    'seq': 19,                        'lat': 40.438185923097,                        'lon': -79.977653302922,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 20,                        'lat': 40.438112431006,                        'lon': -79.975984968915,                        'typ': 'S',                        'stpid': '3250',                        'stpnm': 'Fifth Ave at Seneca St',                        'pdist': 7454.0                },                {                    'seq': 21,                        'lat': 40.43797818236,                        'lon': -79.974548949073,                        'typ': 'S',                        'stpid': '3251',                        'stpnm': '5th Ave at Moulrie St',                        'pdist': 7849.0                },                {                    'seq': 22,                        'lat': 40.437889166532,                        'lon': -79.97340458863,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 23,                        'lat': 40.437725849943,                        'lon': -79.972862782409,                        'typ': 'S',                        'stpid': '3252',                        'stpnm': 'Fifth Ave at Birmingham Bridge',                        'pdist': 8344.0                },                {                    'seq': 24,                        'lat': 40.437214433778,                        'lon': -79.971299708996,                        'typ': 'S',                        'stpid': '2556',                        'stpnm': '5th Ave at #2358',                        'pdist': 8828.0                },                {                    'seq': 25,                        'lat': 40.43686293456,                        'lon': -79.9702225,                        'typ': 'S',                        'stpid': '2557',                        'stpnm': '5th Ave at #2410',                        'pdist': 9167.0                },                {                    'seq': 26,                        'lat': 40.436434221124,                        'lon': -79.968918946419,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 27,                        'lat': 40.436409723131,                        'lon': -79.968758013878,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 28,                        'lat': 40.436405101253,                        'lon': -79.968582166668,                        'typ': 'S',                        'stpid': '2558',                        'stpnm': 'Fifth Ave opp Brenham St',                        'pdist': 9646.0                },                {                    'seq': 29,                        'lat': 40.436421433251,                        'lon': -79.968040360447,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 30,                        'lat': 40.436425516249,                        'lon': -79.966962112423,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 31,                        'lat': 40.436478595211,                        'lon': -79.966511501309,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 32,                        'lat': 40.436580670019,                        'lon': -79.966243280407,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 33,                        'lat': 40.436727657471,                        'lon': -79.965996517178,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 34,                        'lat': 40.436850146768,                        'lon': -79.965846313473,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 35,                        'lat': 40.436988967702,                        'lon': -79.965728296276,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 36,                        'lat': 40.437119622437,                        'lon': -79.965658558842,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 37,                        'lat': 40.437262131201,                        'lon': -79.965561438496,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 38,                        'lat': 40.437429531855,                        'lon': -79.965362955029,                        'typ': 'S',                        'stpid': '19850',                        'stpnm': '5th Ave at Maurice St',                        'pdist': 10672.0                },                {                    'seq': 39,                        'lat': 40.437552019874,                        'lon': -79.965164471562,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 40,                        'lat': 40.437645927204,                        'lon': -79.965003539021,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 41,                        'lat': 40.437735751483,                        'lon': -79.964719224866,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 42,                        'lat': 40.437787848263,                        'lon': -79.964154864419,                        'typ': 'S',                        'stpid': '2561',                        'stpnm': 'Fifth Ave at Craft Ave',                        'pdist': 11060.0                },                {                    'seq': 43,                        'lat': 40.437836843236,                        'lon': -79.96390810119,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 44,                        'lat': 40.437951164701,                        'lon': -79.963613058198,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 45,                        'lat': 40.438171641264,                        'lon': -79.963285828698,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 46,                        'lat': 40.438318625238,                        'lon': -79.963103438485,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 47,                        'lat': 40.438592177888,                        'lon': -79.962744022477,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 48,                        'lat': 40.439010601112,                        'lon': -79.962188666667,                        'typ': 'S',                        'stpid': '2562',                        'stpnm': 'Fifth Ave at Halket St',                        'pdist': 11785.0                },                {                    'seq': 49,                        'lat': 40.440680195665,                        'lon': -79.959905053573,                        'typ': 'S',                        'stpid': '17671',                        'stpnm': 'Fifth Ave past McKee Pl',                        'pdist': 12649.0                },                {                    'seq': 50,                        'lat': 40.441172012068,                        'lon': -79.959239533731,                        'typ': 'S',                        'stpid': '2564',                        'stpnm': '5th Ave at Meyran Ave',                        'pdist': 12896.0                },                {                    'seq': 51,                        'lat': 40.442085267612,                        'lon': -79.957992499998,                        'typ': 'S',                        'stpid': '2565',                        'stpnm': 'Fifth Ave at Oakland Ave',                        'pdist': 13411.0                },                {                    'seq': 52,                        'lat': 40.442990015366,                        'lon': -79.956747457672,                        'typ': 'S',                        'stpid': '2566',                        'stpnm': 'Fifth Ave opp Thackeray Ave',                        'pdist': 13889.0                },                {                    'seq': 53,                        'lat': 40.444335993965,                        'lon': -79.954881824734,                        'typ': 'S',                        'stpid': '2567',                        'stpnm': 'Fifth Ave at Bigelow Blvd',                        'pdist': 14549.0                },                {                    'seq': 54,                        'lat': 40.444983369978,                        'lon': -79.953932269848,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 55,                        'lat': 40.445546749707,                        'lon': -79.953202708996,                        'typ': 'S',                        'stpid': '3253',                        'stpnm': 'Fifth Ave opp Tennyson Ave',                        'pdist': 15210.0                },                {                    'seq': 56,                        'lat': 40.44630592845,                        'lon': -79.952111093253,                        'typ': 'S',                        'stpid': '3254',                        'stpnm': 'Fifth Ave at Bellefield Ave',                        'pdist': 15640.0                },                {                    'seq': 57,                        'lat': 40.446623373397,                        'lon': -79.951609216288,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 58,                        'lat': 40.446721350429,                        'lon': -79.951314173296,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 59,                        'lat': 40.446770338891,                        'lon': -79.950579248026,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 60,                        'lat': 40.446835656786,                        'lon': -79.949640474871,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 61,                        'lat': 40.446913221703,                        'lon': -79.948980651453,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 62,                        'lat': 40.447374526992,                        'lon': -79.94917913492,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 63,                        'lat': 40.447762347572,                        'lon': -79.949420533732,                        'typ': 'S',                        'stpid': '2571',                        'stpnm': 'Craig St at Park Plaza',                        'pdist': 16868.0                },                {                    'seq': 64,                        'lat': 40.449607534262,                        'lon': -79.950570087302,                        'typ': 'S',                        'stpid': '2572',                        'stpnm': 'Craig St at Bayard St',                        'pdist': 17647.0                },                {                    'seq': 65,                        'lat': 40.451872385571,                        'lon': -79.951959353173,                        'typ': 'S',                        'stpid': '2573',                        'stpnm': 'Craig St at Centre Ave',                        'pdist': 18474.0                },                {                    'seq': 66,                        'lat': 40.452173174704,                        'lon': -79.951001553571,                        'typ': 'S',                        'stpid': '2574',                        'stpnm': 'Centre Ave at Melwood Ave',                        'pdist': 18783.0                },                {                    'seq': 67,                        'lat': 40.452535756554,                        'lon': -79.949852615741,                        'typ': 'S',                        'stpid': '2575',                        'stpnm': 'Centre Ave at Neville St',                        'pdist': 19133.0                },                {                    'seq': 68,                        'lat': 40.452783201296,                        'lon': -79.94906657475,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 69,                        'lat': 40.453252630247,                        'lon': -79.94732313889,                        'typ': 'S',                        'stpid': '8198',                        'stpnm': 'Centre Ave opp Millvale Ave',                        'pdist': 19947.0                },                {                    'seq': 70,                        'lat': 40.454097103115,                        'lon': -79.944541689153,                        'typ': 'S',                        'stpid': '8199',                        'stpnm': 'Centre Ave at Morewood Ave',                        'pdist': 20705.0                },                {                    'seq': 71,                        'lat': 40.45498568797,                        'lon': -79.941522513889,                        'typ': 'S',                        'stpid': '8201',                        'stpnm': 'Centre Ave opp Cypress St',                        'pdist': 21579.0                },                {                    'seq': 72,                        'lat': 40.455500688037,                        'lon': -79.939839104496,                        'typ': 'S',                        'stpid': '8202',                        'stpnm': 'Centre Ave at Shadyside Hospital',                        'pdist': 22100.0                },                {                    'seq': 73,                        'lat': 40.455849760816,                        'lon': -79.938622420634,                        'typ': 'S',                        'stpid': '8203',                        'stpnm': 'Centre Ave at Aiken Ave',                        'pdist': 22425.0                },                {                    'seq': 74,                        'lat': 40.456481021498,                        'lon': -79.936516138226,                        'typ': 'S',                        'stpid': '8204',                        'stpnm': 'Centre Ave at Graham St',                        'pdist': 23063.0                },                {                    'seq': 75,                        'lat': 40.457026769981,                        'lon': -79.934688511242,                        'typ': 'S',                        'stpid': '8205',                        'stpnm': 'Centre Ave at Negley Ave',                        'pdist': 23629.0                },                {                    'seq': 76,                        'lat': 40.457233818562,                        'lon': -79.933969974518,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 77,                        'lat': 40.457315453208,                        'lon': -79.934012889862,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 78,                        'lat': 40.457617500534,                        'lon': -79.934141635895,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 79,                        'lat': 40.4576909713,                        'lon': -79.934120178223,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 80,                        'lat': 40.457833830893,                        'lon': -79.93407189846,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 81,                        'lat': 40.457972608492,                        'lon': -79.933980703354,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 82,                        'lat': 40.458263705021,                        'lon': -79.933821595899,                        'typ': 'S',                        'stpid': '8206',                        'stpnm': 'Centre Ave at Baum Blvd',                        'pdist': 24173.0                },                {                    'seq': 83,                        'lat': 40.460559766618,                        'lon': -79.932419593254,                        'typ': 'S',                        'stpid': '8207',                        'stpnm': 'Negley Ave at Friendship Ave',                        'pdist': 25096.0                },                {                    'seq': 84,                        'lat': 40.461936270313,                        'lon': -79.931569697751,                        'typ': 'S',                        'stpid': '8208',                        'stpnm': 'Negley Ave opp Coral St',                        'pdist': 25658.0                },                {                    'seq': 85,                        'lat': 40.463411168093,                        'lon': -79.930680564813,                        'typ': 'S',                        'stpid': '8209',                        'stpnm': 'Negley Ave at Penn Ave',                        'pdist': 26170.0                },                {                    'seq': 86,                        'lat': 40.464580433069,                        'lon': -79.929992666666,                        'typ': 'S',                        'stpid': '8210',                        'stpnm': 'Negley Ave past Broad St',                        'pdist': 26719.0                },                {                    'seq': 87,                        'lat': 40.465522766353,                        'lon': -79.92940348016,                        'typ': 'S',                        'stpid': '8211',                        'stpnm': 'Negley Ave at Rural St',                        'pdist': 27092.0                },                {                    'seq': 88,                        'lat': 40.466404180816,                        'lon': -79.92886335582,                        'typ': 'S',                        'stpid': '8212',                        'stpnm': 'Negley Ave ar Rippey St',                        'pdist': 27447.0                },                {                    'seq': 89,                        'lat': 40.467506928467,                        'lon': -79.928204271163,                        'typ': 'S',                        'stpid': '8213',                        'stpnm': 'Negley Ave at East Liberty Blvd',                        'pdist': 27886.0                },                {                    'seq': 90,                        'lat': 40.468731197382,                        'lon': -79.927462273807,                        'typ': 'S',                        'stpid': '8214',                        'stpnm': 'Negley Ave opp Black St',                        'pdist': 28331.0                },                {                    'seq': 91,                        'lat': 40.46963876613,                        'lon': -79.927062999999,                        'typ': 'S',                        'stpid': '8215',                        'stpnm': 'Negley Ave at Hays St',                        'pdist': 28730.0                },                {                    'seq': 92,                        'lat': 40.471479297677,                        'lon': -79.926241917988,                        'typ': 'S',                        'stpid': '8216',                        'stpnm': 'Negley Ave at Stanton Ave',                        'pdist': 29406.0                },                {                    'seq': 93,                        'lat': 40.472400599317,                        'lon': -79.925819333332,                        'typ': 'S',                        'stpid': '19011',                        'stpnm': 'Negley Ave at Jackson St (nearside)',                        'pdist': 29798.0                },                {                    'seq': 94,                        'lat': 40.474056760653,                        'lon': -79.925098488757,                        'typ': 'S',                        'stpid': '8218',                        'stpnm': 'Negley Ave at Wellesley Ave (nearside)',                        'pdist': 30393.0                },                {                    'seq': 95,                        'lat': 40.475284432494,                        'lon': -79.924568166667,                        'typ': 'S',                        'stpid': '8219',                        'stpnm': 'Negley Ave at Hampton St',                        'pdist': 30871.0                },                {                    'seq': 96,                        'lat': 40.477192181503,                        'lon': -79.923721638226,                        'typ': 'S',                        'stpid': '8220',                        'stpnm': 'Negley Ave at Bryant St',                        'pdist': 31555.0                },                {                    'seq': 97,                        'lat': 40.476467099098,                        'lon': -79.921844499999,                        'typ': 'S',                        'stpid': '8221',                        'stpnm': 'Bryant St at Mellon St',                        'pdist': 32179.0                },                {                    'seq': 98,                        'lat': 40.476112265783,                        'lon': -79.920898499998,                        'typ': 'S',                        'stpid': '8222',                        'stpnm': 'Bryant St at Clair St',                        'pdist': 32460.0                },                {                    'seq': 99,                        'lat': 40.47568225493,                        'lon': -79.91977944643,                        'typ': 'S',                        'stpid': '3280',                        'stpnm': 'Bryant St at Euclid Ave',                        'pdist': 32750.0                },                {                    'seq': 100,                        'lat': 40.4770659324,                        'lon': -79.918991166667,                        'typ': 'S',                        'stpid': '3281',                        'stpnm': 'Euclid Ave at Callowhill St',                        'pdist': 33306.0                },                {                    'seq': 101,                        'lat': 40.479059862173,                        'lon': -79.917793949075,                        'typ': 'S',                        'stpid': '3282',                        'stpnm': 'Euclid Ave at Bunkerhill St',                        'pdist': 34080.0                },                {                    'seq': 102,                        'lat': 40.479112907462,                        'lon': -79.917745669313,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 103,                        'lat': 40.479485912059,                        'lon': -79.918699346557,                        'typ': 'W',                        'pdist': 0.0                },                {                    'seq': 104,                        'lat': 40.479130917836,                        'lon': -79.918946109786,                        'typ': 'S',                        'stpid': '3118',                        'stpnm': 'St Clair St at Bunkerhill Apts',                        'pdist': 34633.0                }                ]            }            ]        }    }";

    private final static PatternResponse patternResponse;
    private final static List<Ptr> patterns;

    static {
        Gson gson = new GsonBuilder().create();
        patternResponse = gson.fromJson(json, PatternResponse.class);
        patterns = patternResponse.getPatternResponse().getPtr();
    }

    public static PatApiService getPatApiServiceMock() {
        PatApiService patApiService = mock(PatApiService.class);
        setupService(patApiService);
        return patApiService;
    }

    public static RetrofitPatApi getPatApiMock() {
        RetrofitPatApi patapi = mock(RetrofitPatApi.class);
        setupMock(patapi);
        return patapi;
    }

    public static PatternResponse getPatternResponse() {
        return patternResponse;
    }

    public static List<Ptr> getPatterns() {
        return patterns;
    }

    private static void setupService(PatApiService patApiService) {
        when(patApiService.getPatterns(testRoute1))
                .thenReturn(Flowable.just(patterns));
    }

    private static void setupMock(RetrofitPatApi patapi) {
        when(patapi.getPatterns(testRoute1))
                .thenReturn(Flowable.just(patternResponse));
    }
}
